"""
Terminal icons/emojis for charstyle.

This module provides a collection of widely supported terminal icons and emojis
that should display correctly on most modern terminals.

Most spinners have been found at:
https://raw.githubusercontent.com/sindresorhus/cli-spinners/main/spinners.json

"""

from enum import StrEnum


class Icon(StrEnum):
    """Common terminal icons/emojis that work across most modern terminals."""

    # Status icons
    CHECK = "✓"
    CROSS = "✗"
    WARNING = "⚠"
    INFO = "ℹ"

    # Directional arrows
    ARROW_RIGHT = "→"
    ARROW_LEFT = "←"
    ARROW_UP = "↑"
    ARROW_DOWN = "↓"

    # Common shapes
    CIRCLE = "●"
    SQUARE = "■"
    TRIANGLE = "▲"
    STAR = "★"

    # Weather & elements
    SUN = "☀"
    CLOUD = "☁"
    UMBRELLA = "☂"
    SNOWFLAKE = "❄"

    # Weather cycle
    WEATHER_SUN = "☀️"
    WEATHER_SUN_SMALL_CLOUD = "🌤"
    WEATHER_SUN_CLOUD = "⛅️"
    WEATHER_CLOUD_SUN = "🌥"
    WEATHER_CLOUD = "☁️"
    WEATHER_RAIN = "🌧"
    WEATHER_SNOW = "🌨"
    WEATHER_THUNDERSTORM = "⛈"

    # Globe/Earth
    GLOBE_EUROPE_AFRICA = "🌍"
    GLOBE_AMERICAS = "🌎"
    GLOBE_ASIA_AUSTRALIA = "🌏"

    # Nature
    TREE_EVERGREEN = "🌲"
    TREE_CHRISTMAS = "🎄"

    # Misc
    HEART = "♥"
    MUSIC = "♪"
    RECYCLE = "♻"
    TELEPHONE = "☎"

    # Faces
    SMILE = "☺"
    FROWN = "☹"

    # Monkeys
    MONKEY_SEE_NO_EVIL = "🙈"
    MONKEY_HEAR_NO_EVIL = "🙉"
    MONKEY_SPEAK_NO_EVIL = "🙊"

    # People
    PERSON_WALKING = "🚶"
    PERSON_RUNNING = "🏃"

    # Tech
    MAIL = "✉"
    SCISSORS = "✂"
    PENCIL = "✎"
    KEY = "⚿"

    # Geometric
    BLOCK = "█"
    LIGHT_SHADE = "░"
    MEDIUM_SHADE = "▒"
    DARK_SHADE = "▓"

    # Box drawing - basic
    H_LINE = "─"
    V_LINE = "│"
    TOP_LEFT = "┌"
    TOP_RIGHT = "┐"
    BOTTOM_LEFT = "└"
    BOTTOM_RIGHT = "┘"

    # Box drawing - extended
    H_LINE_HEAVY = "━"
    V_LINE_HEAVY = "┃"
    TOP_LEFT_HEAVY = "┏"
    TOP_RIGHT_HEAVY = "┓"
    BOTTOM_LEFT_HEAVY = "┗"
    BOTTOM_RIGHT_HEAVY = "┛"
    BOX_CROSS = "┼"
    BOX_CROSS_HEAVY = "╋"
    T_RIGHT = "├"
    T_LEFT = "┤"
    T_DOWN = "┬"
    T_UP = "┴"

    # Spinners - Braille patterns
    SPIN_BRAILLE_1 = "⠋"
    SPIN_BRAILLE_2 = "⠙"
    SPIN_BRAILLE_3 = "⠹"
    SPIN_BRAILLE_4 = "⠸"
    SPIN_BRAILLE_5 = "⠼"
    SPIN_BRAILLE_6 = "⠴"
    SPIN_BRAILLE_7 = "⠦"
    SPIN_BRAILLE_8 = "⠧"
    SPIN_BRAILLE_9 = "⠇"
    SPIN_BRAILLE_10 = "⠏"

    # Spinners - Line patterns
    SPIN_LINE_1 = "|"
    SPIN_LINE_2 = "/"
    SPIN_LINE_3 = "-"
    SPIN_LINE_4 = "\\"

    # Spinners - Dot patterns
    SPIN_DOT_1 = "⣾"
    SPIN_DOT_2 = "⣽"
    SPIN_DOT_3 = "⣻"
    SPIN_DOT_4 = "⢿"
    SPIN_DOT_5 = "⡿"
    SPIN_DOT_6 = "⣟"
    SPIN_DOT_7 = "⣯"
    SPIN_DOT_8 = "⣷"

    # Spinners - Arrow patterns
    SPIN_ARROW_1 = "←"
    SPIN_ARROW_2 = "↖"
    SPIN_ARROW_3 = "↑"
    SPIN_ARROW_4 = "↗"
    SPIN_ARROW_5 = "→"
    SPIN_ARROW_6 = "↘"
    SPIN_ARROW_7 = "↓"
    SPIN_ARROW_8 = "↙"

    # Spinners - Emoji Arrow patterns
    SPIN_EMOJI_ARROW_1 = "⬅️"
    SPIN_EMOJI_ARROW_2 = "↖️"
    SPIN_EMOJI_ARROW_3 = "⬆️"
    SPIN_EMOJI_ARROW_4 = "↗️"
    SPIN_EMOJI_ARROW_5 = "➡️"
    SPIN_EMOJI_ARROW_6 = "↘️"
    SPIN_EMOJI_ARROW_7 = "⬇️"
    SPIN_EMOJI_ARROW_8 = "↙️"

    # Spinners - Clock patterns
    SPIN_CLOCK_1 = "🕛"
    SPIN_CLOCK_2 = "🕐"
    SPIN_CLOCK_3 = "🕑"
    SPIN_CLOCK_4 = "🕒"
    SPIN_CLOCK_5 = "🕓"
    SPIN_CLOCK_6 = "🕔"
    SPIN_CLOCK_7 = "🕕"
    SPIN_CLOCK_8 = "🕖"
    SPIN_CLOCK_9 = "🕗"
    SPIN_CLOCK_10 = "🕘"
    SPIN_CLOCK_11 = "🕙"
    SPIN_CLOCK_12 = "🕚"

    # Emotional
    THUMBS_UP = "👍"
    THUMBS_DOWN = "👎"
    CLAP = "👏"
    FIRE = "🔥"

    # Hearts
    HEART_RED = "❤️"
    HEART_YELLOW = "💛"
    HEART_BLUE = "💙"
    HEART_PURPLE = "💜"
    HEART_GREEN = "💚"

    # Celebration
    CONFETTI = "🎊"
    PARTY = "🎉"
    BALLOON = "🎈"
    GIFT = "🎁"

    # Faces & People
    FACE_GRINNING = "😀"
    FACE_GRINNING_WITH_BIG_EYES = "😃"
    FACE_GRINNING_WITH_SMILE = "😄"
    FACE_BEAMING = "😁"
    FACE_GRINNING_WITH_SWEAT = "😅"
    FACE_LAUGHING = "😆"
    FACE_JOY = "😂"
    FACE_ROFL = "🤣"
    FACE_WINK = "😉"
    FACE_BLUSH = "😊"
    FACE_YUM = "😋"
    FACE_SUNGLASSES = "😎"
    FACE_DISGUISED = "🥸"
    FACE_HEART_EYES = "😍"
    FACE_LOVE = "🥰"
    FACE_KISS = "😘"
    FACE_MELTING = "🫠"
    FACE_THINKING = "🤔"
    FACE_SHUSHING = "🤫"
    FACE_HAND_OVER_MOUTH = "🤭"
    FACE_SALUTING = "🫡"
    FACE_HUGGING = "🤗"
    PEOPLE_HUGGING = "🫂"
    HAND_WAVE = "👋"
    HAND_OK = "👌"
    HAND_THUMBS_UP = "👍"
    HAND_THUMBS_DOWN = "👎"
    HAND_CLAP = "👏"
    HAND_RAISED = "🙌"
    HAND_HEART = "🫶"
    HAND_VICTORY = "✌️"
    HAND_CROSSED_FINGERS = "🤞"
    HAND_LOVE_YOU = "🤟"
    HAND_CALL_ME = "🤙"
    HAND_HANDSHAKE = "🤝"
    HAND_PRAY = "🙏"

    # Nature & Animals
    ANIMAL_DOG = "🐶"
    ANIMAL_CAT = "🐱"
    ANIMAL_MOUSE = "🐭"
    ANIMAL_HAMSTER = "🐹"
    ANIMAL_RABBIT = "🐰"
    ANIMAL_FOX = "🦊"
    ANIMAL_BEAR = "🐻"
    ANIMAL_PANDA = "🐼"
    ANIMAL_KOALA = "🐨"
    ANIMAL_LION = "🦁"
    ANIMAL_TIGER = "🐯"
    ANIMAL_COW = "🐮"
    ANIMAL_PIG = "🐷"
    ANIMAL_FROG = "🐸"
    ANIMAL_MONKEY = "🐵"
    ANIMAL_DUCK = "🦆"
    ANIMAL_EAGLE = "🦅"
    ANIMAL_OWL = "🦉"
    ANIMAL_BAT = "🦇"
    ANIMAL_BEE = "🐝"
    ANIMAL_BUTTERFLY = "🦋"
    ANIMAL_SNAIL = "🐌"
    ANIMAL_LADYBUG = "🐞"
    ANIMAL_SCORPION = "🦂"
    ANIMAL_SQUID = "🦑"
    ANIMAL_OCTOPUS = "🐙"
    ANIMAL_SHRIMP = "🦐"
    ANIMAL_TROPICAL_FISH = "🐠"
    ANIMAL_FISH = "🐟"
    ANIMAL_BLOWFISH = "🐡"
    ANIMAL_SHARK = "🦈"
    ANIMAL_DOLPHIN = "🐬"
    ANIMAL_WHALE = "🐳"
    ANIMAL_CROCODILE = "🐊"
    ANIMAL_T_REX = "🦖"
    ANIMAL_SAUROPOD = "🦕"

    # Food & Drink
    FOOD_APPLE_GREEN = "🍏"
    FOOD_APPLE_RED = "🍎"
    FOOD_PEAR = "🍐"
    FOOD_ORANGE = "🍊"
    FOOD_LEMON = "🍋"
    FOOD_BANANA = "🍌"
    FOOD_WATERMELON = "🍉"
    FOOD_GRAPES = "🍇"
    FOOD_STRAWBERRY = "🍓"
    FOOD_BLUEBERRIES = "🫐"
    FOOD_MELON = "🍈"
    FOOD_CHERRIES = "🍒"
    FOOD_PEACH = "🍑"
    FOOD_MANGO = "🥭"
    FOOD_PINEAPPLE = "🍍"
    FOOD_COCONUT = "🥥"
    FOOD_KIWI = "🥝"
    FOOD_TOMATO = "🍅"
    FOOD_EGGPLANT = "🍆"
    FOOD_AVOCADO = "🥑"
    FOOD_BROCCOLI = "🥦"
    FOOD_LEAFY_GREEN = "🥬"
    FOOD_CUCUMBER = "🥒"
    FOOD_HOT_PEPPER = "🌶"
    FOOD_BELL_PEPPER = "🫑"
    FOOD_POTATO = "🥔"
    FOOD_FRIES = "🍟"
    FOOD_PIZZA = "🍕"
    FOOD_HOTDOG = "🌭"
    FOOD_HAMBURGER = "🍔"
    FOOD_CHICKEN = "🍗"
    FOOD_MEAT_ON_BONE = "🍖"
    FOOD_STEAK = "🥩"
    FOOD_SUSHI = "🍣"
    FOOD_BENTO = "🍱"
    FOOD_RICE_CRACKER = "🍘"
    FOOD_RICE = "🍚"
    FOOD_RAMEN = "🍜"
    FOOD_SPAGHETTI = "🍝"
    FOOD_SWEET_POTATO = "🍠"
    FOOD_CROISSANT = "🥐"
    FOOD_BAGEL = "🥯"
    FOOD_BREAD = "🍞"

    # Activities & Objects
    SPORT_SOCCER = "⚽️"
    SPORT_BASKETBALL = "🏀"
    SPORT_FOOTBALL = "🏈"
    SPORT_BASEBALL = "⚾️"
    SPORT_SOFTBALL = "🥎"
    SPORT_TENNIS = "🎾"
    SPORT_VOLLEYBALL = "🏐"
    SPORT_RUGBY = "🏉"
    SPORT_POOL_8_BALL = "🎱"
    SPORT_PING_PONG = "🏓"
    SPORT_BADMINTON = "🏸"
    SPORT_HOCKEY = "🏒"
    SPORT_GOAL_NET = "🥅"
    SPORT_GOLF = "⛳️"
    SPORT_FISHING = "🎣"
    SPORT_ARCHERY = "🏹"
    SPORT_RUNNING_SHIRT = "🎽"
    SPORT_SKI = "🎿"
    SPORT_SLED = "🛷"
    SPORT_CURLING_STONE = "🥌"
    GAME_VIDEO_GAME = "🎮"
    GAME_JOYSTICK = "🕹"
    GAME_DICE = "🎲"
    GAME_DART = "🎯"
    GAME_PERFORMING_ARTS = "🎭"
    ART_ARTIST_PALETTE = "🎨"
    ART_FILM_FRAMES = "🎬"
    MUSIC_MICROPHONE = "🎤"
    MUSIC_HEADPHONE = "🎧"
    MUSIC_MUSICAL_SCORE = "🎼"
    MUSIC_PIANO = "🎹"
    MUSIC_DRUM = "🪘"
    MUSIC_SAXOPHONE = "🎷"
    MUSIC_TRUMPET = "🎺"
    MUSIC_GUITAR = "🎸"
    MUSIC_BANJO = "🪕"
    MUSIC_VIOLIN = "🎻"
    TECH_MOBILE_PHONE = "📱"
    TECH_LAPTOP = "💻"
    TECH_DESKTOP = "🖥"
    TECH_PRINTER = "🖨"
    TECH_KEYBOARD = "⌨️"
    TECH_MOUSE = "🖱"
    TECH_COMPRESSION = "🗜"

    # Technical & Computing
    TECH_WATCH = "⌚️"
    TECH_SMARTPHONE = "📱"
    TECH_CALLING = "📲"
    TECH_COMPUTER = "💻"
    TECH_COMPUTER_KEYBOARD = "⌨️"
    TECH_DESKTOP_COMPUTER = "🖥"
    TECH_COMPUTER_PRINTER = "🖨"
    TECH_COMPUTER_MOUSE = "🖱"
    TECH_TRACKBALL = "🖲"
    TECH_JOYSTICK = "🕹"
    TECH_HARD_DISK = "🗜"
    TECH_MINIDISC = "💽"
    TECH_VHS = "📼"
    TECH_CAMERA = "📷"
    TECH_CAMERA_WITH_FLASH = "📸"
    TECH_VIDEO_CAMERA = "📹"
    TECH_MOVIE_CAMERA = "🎥"
    TECH_FILM_PROJECTOR = "📽"
    TECH_FILM_FRAMES = "🎞"
    TECH_TELEPHONE = "📞"
    TECH_TELEPHONE_RECEIVER = "☎️"
    TECH_PAGER = "📟"
    TECH_FAX_MACHINE = "📠"
    TECH_TV = "📺"
    TECH_RADIO = "📻"
    TECH_STUDIO_MICROPHONE = "🎙"
    TECH_LEVEL_SLIDER = "🎚"
    TECH_CONTROL_KNOBS = "🎛"
    TECH_STOPWATCH = "⏱"
    TECH_TIMER_CLOCK = "⏲"
    TECH_ALARM_CLOCK = "⏰"
    TECH_MANTELPIECE_CLOCK = "🕰"
    TECH_HOURGLASS = "⌛️"
    TECH_HOURGLASS_WITH_FLOWING_SAND = "⏳"
    TECH_SATELLITE_ANTENNA = "📡"
    TECH_BATTERY = "🔋"
    TECH_ELECTRIC_PLUG = "🔌"
    TECH_LIGHT_BULB = "💡"
    TECH_FLASHLIGHT = "🔦"
    TECH_CANDLE = "🕯"
    TECH_DIYA_LAMP = "🪔"

    # Travel & Places
    VEHICLE_CAR = "🚗"
    VEHICLE_TAXI = "🚕"
    VEHICLE_SUV = "🚙"
    VEHICLE_BUS = "🚌"
    VEHICLE_TROLLEYBUS = "🚎"
    VEHICLE_RACING_CAR = "🏎"
    VEHICLE_POLICE_CAR = "🚓"
    VEHICLE_AMBULANCE = "🚑"
    VEHICLE_FIRE_ENGINE = "🚒"
    VEHICLE_MINIBUS = "🚐"
    VEHICLE_PICKUP_TRUCK = "🛻"
    VEHICLE_DELIVERY_TRUCK = "🚚"
    VEHICLE_ARTICULATED_LORRY = "🚛"
    VEHICLE_TRACTOR = "🚜"
    VEHICLE_KICK_SCOOTER = "🛴"
    VEHICLE_BICYCLE = "🚲"
    VEHICLE_MOTOR_SCOOTER = "🛵"
    VEHICLE_MOTORCYCLE = "🏍"
    VEHICLE_AUTO_RICKSHAW = "🛺"
    VEHICLE_POLICE_LIGHT = "🚨"
    VEHICLE_ONCOMING_POLICE_CAR = "🚔"
    VEHICLE_ONCOMING_BUS = "🚍"
    VEHICLE_ONCOMING_AUTOMOBILE = "🚘"
    VEHICLE_ONCOMING_TAXI = "🚖"
    VEHICLE_AERIAL_TRAMWAY = "🚡"
    VEHICLE_MOUNTAIN_CABLEWAY = "🚠"
    VEHICLE_SUSPENSION_RAILWAY = "🚟"
    VEHICLE_RAILWAY_CAR = "🚃"
    VEHICLE_TRAM_CAR = "🚋"
    VEHICLE_MONORAIL = "🚝"
    VEHICLE_HIGH_SPEED_TRAIN = "🚄"
    VEHICLE_BULLET_TRAIN = "🚅"
    VEHICLE_LIGHT_RAIL = "🚈"
    VEHICLE_LOCOMOTIVE = "🚂"
    VEHICLE_TRAIN = "🚆"
    VEHICLE_METRO = "🚇"
    VEHICLE_TRAM = "🚊"
    VEHICLE_STATION = "🚉"
    VEHICLE_HELICOPTER = "🚁"
    VEHICLE_SMALL_AIRPLANE = "🛩"
    VEHICLE_AIRPLANE = "✈️"
    VEHICLE_AIRPLANE_DEPARTURE = "🛫"
    VEHICLE_AIRPLANE_ARRIVAL = "🛬"
    VEHICLE_ROCKET = "🚀"
    VEHICLE_SATELLITE = "🛰"

    # Symbols
    SYMBOL_RED_HEART = "❤️"
    SYMBOL_ORANGE_HEART = "🧡"
    SYMBOL_YELLOW_HEART = "💛"
    SYMBOL_GREEN_HEART = "💚"
    SYMBOL_BLUE_HEART = "💙"
    SYMBOL_PURPLE_HEART = "💜"
    SYMBOL_BLACK_HEART = "🖤"
    SYMBOL_WHITE_HEART = "🤍"
    SYMBOL_BROWN_HEART = "🤎"
    SYMBOL_BROKEN_HEART = "💔"
    SYMBOL_TWO_HEARTS = "💕"
    SYMBOL_REVOLVING_HEARTS = "💞"
    SYMBOL_BEATING_HEART = "💓"
    SYMBOL_GROWING_HEART = "💗"
    SYMBOL_SPARKLING_HEART = "💖"
    SYMBOL_HEART_WITH_ARROW = "💘"
    SYMBOL_HEART_WITH_RIBBON = "💝"
    SYMBOL_HEART_DECORATION = "💟"
    SYMBOL_PEACE = "☮️"
    SYMBOL_LATIN_CROSS = "✝️"
    SYMBOL_STAR_AND_CRESCENT = "☪️"
    SYMBOL_OM = "🕉"
    SYMBOL_WHEEL_OF_DHARMA = "☸️"
    SYMBOL_STAR_OF_DAVID = "✡️"
    SYMBOL_DOTTED_SIX_POINTED_STAR = "🔯"
    SYMBOL_MENORAH = "🕎"
    SYMBOL_YIN_YANG = "☯️"
    SYMBOL_ORTHODOX_CROSS = "☦️"
    SYMBOL_PLACE_OF_WORSHIP = "🛐"
    SYMBOL_OPHIUCHUS = "⛎"
    SYMBOL_ARIES = "♈️"
    SYMBOL_TAURUS = "♉️"
    SYMBOL_GEMINI = "♊️"
    SYMBOL_CANCER = "♋️"
    SYMBOL_LEO = "♌️"
    SYMBOL_VIRGO = "♍️"
    SYMBOL_LIBRA = "♎️"
    SYMBOL_SCORPIO = "♏️"
    SYMBOL_SAGITTARIUS = "♐️"
    SYMBOL_CAPRICORN = "♑️"
    SYMBOL_AQUARIUS = "♒️"
    SYMBOL_PISCES = "♓️"

    # Flags
    FLAG_WHITE = "🏳️"
    FLAG_BLACK = "🏴"
    FLAG_PIRATE = "🏴‍☠️"
    FLAG_CHECKERED = "🏁"
    FLAG_TRIANGULAR = "🚩"
    FLAG_RAINBOW = "🏳️‍🌈"
    FLAG_TRANSGENDER = "🏳️‍⚧️"
    FLAG_USA = "🇺🇸"
    FLAG_UK = "🇬🇧"
    FLAG_CANADA = "🇨🇦"
    FLAG_MEXICO = "🇲🇽"
    FLAG_BRAZIL = "🇧🇷"
    FLAG_JAPAN = "🇯🇵"
    FLAG_SOUTH_KOREA = "🇰🇷"
    FLAG_CHINA = "🇨🇳"
    FLAG_INDIA = "🇮🇳"
    FLAG_RUSSIA = "🇷🇺"
    FLAG_FRANCE = "🇫🇷"
    FLAG_GERMANY = "🇩🇪"
    FLAG_ITALY = "🇮🇹"
    FLAG_SPAIN = "🇪🇸"
    FLAG_AUSTRALIA = "🇦🇺"


def print_all_icons() -> None:
    """Print all available icons with their names."""
    for name, icon in [(name, icon.value) for name, icon in Icon.__members__.items()]:
        print(f"{name}: {icon}")


def get_icon(name: str) -> str:
    """
    Get an icon by its name.

    Args:
        name (str): The name of the icon to retrieve

    Returns:
        str: The icon character

    Raises:
        ValueError: If the icon name is not found
    """
    try:
        return Icon[name].value
    except KeyError as err:
        raise ValueError(
            f"Icon '{name}' not found. Available icons: {', '.join(Icon.__members__.keys())}"
        ) from err
